/*****************************************************************

Copyright (c) 1996-2000 the kicker authors. See file AUTHORS.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#ifndef __panelbutton_h__
#define __panelbutton_h__

#include <kurldrag.h>

#include "panelbuttonbase.h"

class QPopupMenu;
class KConfigGroup;
class PanelButton: public PanelButtonBase
{
    Q_OBJECT

public:
    PanelButton( QWidget* parent, const char* name );

    virtual void configure();
    virtual void saveConfig( KConfigGroup& ) const {}

    virtual void properties() {};

signals:
    void requestSave();

protected:
    virtual QString tileName() = 0;

protected slots:
    void slotIconChanged(int group);
};

/**
 * Base class for panelbuttons which can popup a menu
 */
class PanelPopupButton : public PanelButton
{
    Q_OBJECT

public:
    PanelPopupButton(QWidget *parent=0, const char *name=0);

    void setPopup(QPopupMenu *);
    QPopupMenu *popup() const;

    bool eventFilter(QObject *, QEvent *);

protected:
    virtual void initPopup() {};

protected slots:
    virtual void slotExecMenu();

private:
    QPopupMenu *_popup;
    bool _pressedDuringPopup;
};

class PanelDrag : public KURLDrag
{
public:
   PanelDrag(const KURL::List &uris, QWidget *dragSource)
     : KURLDrag(uris, dragSource, 0)
      { PanelButton::setZoomEnabled(false); }

   ~PanelDrag()
      { PanelButton::setZoomEnabled(true); }
};


#endif // __panelbutton_h__
