/*****************************************************************

Copyright (c) 1996-2000 the kicker authors. See file AUTHORS.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#ifndef __pluginmgr_h__
#define __pluginmgr_h__

#include <qobject.h>
#include <qdict.h>
#include <qstringlist.h>

class AppletContainer;
class ExtensionContainer;
class AppletInfo;
class QStringList;
class QPopupMenu;

class PluginManager : public QObject
{
    Q_OBJECT

public:
    static PluginManager* pluginManager();

    QValueList<AppletInfo> applets() const;
    QValueList<AppletInfo> extensions() const;

    AppletContainer* createAppletContainer(
       const QString& desktopFile, bool isStartup, const QString& configFile, QPopupMenu* opMenu, QWidget* parent );
    ExtensionContainer* createExtensionContainer(
       const QString& desktopFile, bool isStartup, const QString& configFile );

protected:

    enum InternalLoadingPolicy { Trusted = 0, Startup, All };

    PluginManager();
    void configure();
    QValueList<AppletInfo> plugins( const QStringList& desktopFiles ) const;
    bool trustedPlugin( const AppletInfo&, bool isStartup );

    InternalLoadingPolicy    loadingPolicy;
    QStringList              trustedApplets;
    QStringList              trustedExtensions;
};

#endif
